from .APIClients import LoginSession


class EthosLoginSessionBasedOnAPIKey(LoginSession):
  APIClient = None
  apikey = None
  currentAuthKey = None
  def __init__(self, APIClient, apikey):
    self.APIClient = APIClient
    self.apikey = apikey

    self._getNewAuthToken()
    if self.currentAuthKey is None:
      raise Exception("Failed to establish login session using APIKey")

  def _getNewAuthToken(self):
    self.currentAuthKey = None
    charset = "UTF-8"

    def injectHeaderFN(headers):
      headers["Accept-Charset"] = charset
      headers["Content-Type"] = "application/x-www-form-urlencoded" + ";charset=" + charset
      headers["Authorization"] = "Bearer " + self.apikey

    result = self.APIClient.sendPostRequest(
      url="/auth",
      data=[],
      loginSession=None,
      injectHeadersFn=injectHeaderFN
    )
    if result.status_code != 200:
      return None

    self.currentAuthKey = result.content.decode(charset)

  def injectHeaders(self, headers):
    headers["Authorization"] = "Bearer " + self.currentAuthKey

  def refresh(self):
    self._getNewAuthToken()
    if self.currentAuthKey is None:
      return False
    return True



import requests
from .Mock import MockClass

class APIClientException(Exception):
  result = None
  def __init__(self, result):
    self.result = result
  def getDescriptionString(self):
    ret = ""
    ret += "Failed API request\n"
    ret += "Request: " + str(self.result.request.method) + ":" + str(self.result.request.url) + "\n"
    ret += "Response: " + str(self.result.status_code) + ":" + self.result.content.decode() + "\n"
    return ret
  def __str__(self):
    return self.getDescriptionString()

class APIClientNotFoundException(APIClientException):
  def __init__(self, result):
    super(APIClientNotFoundException, self).__init__(result)



class APIClientBase():
  mock = None
  baseURL = None

  def __init__(self, baseURL, mock=None):
    if baseURL.endswith("/"):
      raise Exception("baseURL should not contain trailing slash: " + baseURL)

    if mock is None:
      self.mock = MockClass()
    else:
      self.mock = mock

    self.baseURL = baseURL

  def raiseResponseException(self, result):
    if (result.status_code == 404):
      raise APIClientNotFoundException(result)
    raise APIClientException(result)

  def testRegisterNextResponse(self, *args, **kwargs):
    return self.mock.registerNextResponse(*args, **kwargs)
  def testComplete(self):
    return self.mock.testComplete()
  def testGetMockObj(self):
    return self.mock


  def sendRequest(self, reqFn, url, loginSession, data, origin, injectHeadersFn, refreshAttempted=False):
    if self.baseURL == "MOCK":
      return self.mock.returnNextResult(reqFnName=reqFn.__name__, url=url, data=data)

    # url must start with slash
    headers = {}
    if loginSession is not None:
      loginSession.injectHeaders(headers)
    if origin is not None:
      headers["origin"] = origin
    if injectHeadersFn is not None:
      injectHeadersFn(headers)

    result = reqFn(
      url=self.baseURL + url,
      data=data,
      headers=headers
    )
    if result.status_code == 401:
      if refreshAttempted:
        self.raiseResponseException(result)
      if loginSession is None:
        self.raiseResponseException(result)

      if loginSession.refresh(): #Returns true if loginSession refresh succeeded
        self.sendRequest(
          reqFn=reqFn,
          url=url,
          loginSession=loginSession,
          data=data,
          origin=origin,
          injectHeadersFn=injectHeadersFn,
          refreshAttempted=True
        )
      else:
        self.raiseResponseException(result)

    return result

  def sendGetRequest(self, url, loginSession, origin=None, injectHeadersFn=None):
    return self.sendRequest(reqFn=requests.get, url=url, loginSession=loginSession, data=None, origin=origin, injectHeadersFn=injectHeadersFn)

  def sendPostRequest(self, url, loginSession, data, origin=None, injectHeadersFn=None):
    return self.sendRequest(reqFn=requests.post, url=url, loginSession=loginSession, data=data, origin=origin, injectHeadersFn=injectHeadersFn)

  def sendPutRequest(self, url, loginSession, data, origin=None, injectHeadersFn=None):
    return self.sendRequest(reqFn=requests.put, url=url, loginSession=loginSession, data=data, origin=origin, injectHeadersFn=injectHeadersFn)

  def sendDeleteRequest(self, url, loginSession, data=None, origin=None, injectHeadersFn=None):
    return self.sendRequest(reqFn=requests.delete, url=url, loginSession=loginSession, data=data, origin=origin, injectHeadersFn=injectHeadersFn)



from .APIClients import APIClientBase
from .EthosLoginSession import EthosLoginSessionBasedOnAPIKey
from .ResourceWrappers import getResourceWrapper
from .ResourceIterator import ResourceIterator
import json

class EllucianEthosAPIClient(APIClientBase):
  refreshAuthTokenIfRequired = None

  def __init__(self, baseURL, mock=None):
    super().__init__(baseURL=baseURL, mock=mock)

  def getLoginSessionFromAPIKey(self, apiKey):
    return EthosLoginSessionBasedOnAPIKey(APIClient=self, apikey=apiKey)

  #Doc list https://xedocs.ellucian.com/xe-banner-api/ethos_apis/foundation/persons/person_get_guid_v6.html
  def getResource(self, loginSession, resourceName, resourceID, version=None):
    def injectHeaderFN(headers):
      if version is not None:
        headers["Accept"] = "application/vnd.hedtech.integration.v" + version + "+json"

    result = self.sendGetRequest(
      url="/api/" + resourceName + "/" + resourceID,
      loginSession=loginSession,
      injectHeadersFn=injectHeaderFN
    )
    if result.status_code == 404:
      return None
    if result.status_code != 200:
      self.raiseResponseException(result)

    versionReturned = self.getVersionIntFromHeader(result.headers["x-hedtech-media-type"])

    return getResourceWrapper(clientAPIInstance=self, dict=json.loads(result.content), version=versionReturned, resourseName=resourceName)

  def getResourceIterator(self, loginSession, resourceName, version=None, pageSize=25):
    return ResourceIterator(self, loginSession, resourceName, version, pageSize)

  def getVersionIntFromHeader(self, meaidTypeHeaderValue):
    #example: application/vnd.hedtech.integration.v6+json
    requiredStart = "application/vnd.hedtech.integration.v"
    requiredEnd = "+json"
    if not meaidTypeHeaderValue.startswith(requiredStart):
      raise Exception("Could not determine resource version")
    meaidTypeHeaderValue = meaidTypeHeaderValue[len(requiredStart):]
    if not meaidTypeHeaderValue.endswith(requiredEnd):
      raise Exception("Could not determine resource version - header didn't end with " + requiredEnd)
    meaidTypeHeaderValue = meaidTypeHeaderValue[:-len(requiredEnd)]
    return meaidTypeHeaderValue

  def createResource(
    self,
    loginSession,
    resourceName,
    resourceDict,
    version=None
  ):
    if version is None:
      raise Exception("Must supply version when creating resource")
    def injectHeaderFN(headers):
        headers["Accept"] = "application/vnd.hedtech.integration.v" + version + "+json"
        headers["Content-Type"] = "application/vnd.hedtech.integration.v" + version + "+json"

    result = self.sendPostRequest(
      url="/api/" + resourceName,
      loginSession=loginSession,
      injectHeadersFn=injectHeaderFN,
      data=json.dumps(resourceDict)
    )
    if result.status_code != 201:
      self.raiseResponseException(result)

    versionReturned = self.getVersionIntFromHeader(result.headers["x-hedtech-media-type"])

    return getResourceWrapper(clientAPIInstance=self, dict=json.loads(result.content), version=versionReturned, resourseName=resourceName)

  def deleteResource(
    self,
    loginSession,
    resourceName,
    resourceID
  ):
    url = "/api/" + resourceName + "/" + resourceID

    result = self.sendDeleteRequest(
      url=url,
      loginSession=loginSession,
      injectHeadersFn=None
    )
    if result.status_code != 200:
      self.raiseResponseException(result)
